/*
 * Tool name   : ARPScanner
 * Version     : 0.1
 * Description : A tool to scan an IP range based on ARP requests/replies.
 * Author      : Ruben Unteregger
 * Webpage     : http://www.megapanzer.com
 * OS          : Tested on Microsoft Windows XP
 * Todo        : -
 *
 * Changes     : 
 * 
 *
 * License     :
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */



#define HAVE_REMOTE

#include <stdlib.h>
#include <stdio.h>
#include <pcap.h>
#include "ARPScanner.h"

#pragma comment(lib, "wpcap.lib")
#pragma comment(lib, "Ws2_32.lib")






/*
 * Program entry point
 *
 */

int main(int argc, char **argv)
{
  int lRetVal = 0;
  unsigned char lStartIP[16];
  unsigned char lStopIP[16];
  unsigned char lLocalIP[16];
  unsigned char lLocalMAC[6];
  int lIFCNum = 0;
  HANDLE lThreadHandle = INVALID_HANDLE_VALUE;
  DWORD lThreadId = 0;
  SCANPARAMS lScanParams;


  if (argc == 2 && !strcmp(argv[1], "-l"))
  {
    listInterfaces();
  } else if (argc == 6) {
	ZeroMemory(lStartIP, sizeof(lStartIP));
	ZeroMemory(lStopIP, sizeof(lStopIP));
    ZeroMemory(lLocalIP, sizeof(lLocalIP));
    ZeroMemory(lLocalMAC, sizeof(lLocalMAC));
	
    // Interface
    sscanf(argv[1], "%d", &lIFCNum);
    // Local IP
    sscanf(argv[2], "%d.%d.%d.%d", &lLocalIP[0], &lLocalIP[1], &lLocalIP[2], &lLocalIP[3]);
	// Local MAC
    sscanf(argv[3], "%2x-%2x-%2x-%2x-%2x-%2x", &lLocalMAC[0], &lLocalMAC[1], &lLocalMAC[2], 		                                       &lLocalMAC[3], &lLocalMAC[4], &lLocalMAC[5]);
    // Range Start IP
    sscanf(argv[4], "%d.%d.%d.%d", &lStartIP[0], &lStartIP[1], &lStartIP[2], &lStartIP[3]);
    // Range Start IP
    sscanf(argv[5], "%d.%d.%d.%d", &lStopIP[0], &lStopIP[1], &lStopIP[2], &lStopIP[3]);



	printf("IFC Num   : (%s)\t\t\t%d\n", argv[1], lIFCNum);
	printf("Local IP  : (%s)\t%d.%d.%d.%d\n", argv[2], lLocalIP[0], lLocalIP[1], lLocalIP[2], lLocalIP[3]);
	printf("Local MAC : (%s)\t%02x-%02x-%02x-%02x-%02x-%02x\n", argv[3], lLocalMAC[0], lLocalMAC[1], lLocalMAC[2], 
		                                       lLocalMAC[3], lLocalMAC[4], lLocalMAC[5]);
    printf("Start IP  : (%s)\t%d.%d.%.d.%d\n", argv[4], lStartIP[0], lStartIP[1], lStartIP[2], lStartIP[3]);
	printf("Stop  IP  : (%s)\t%d.%d.%.d.%d\n\n\n", argv[5], lStopIP[0], lStopIP[1], lStopIP[2], lStopIP[3]);


	/*
	 * Scan network range.
	 *
	 */
    memcpy(lScanParams.StartIP, lStartIP, 4);
    memcpy(lScanParams.StopIP, lStopIP, 4);
	memcpy(lScanParams.LocalMAC, lLocalMAC, 6);
    memcpy(lScanParams.LocalIP, lLocalIP, 4);
	lScanParams.IFCNum = lIFCNum;

    ScanNetwork(&lScanParams);

  } else {
	  printf("Usage : %s IFC# Local-IP Local-MAC Start-IP Stop-IP \t\tEnum all reachable hosts\n", argv[0]);
      printf("\t%s 1 192.168.0.104 00-12-e9-a3-ce-73 192.168.0.1 192.168.0.254 1\tEnum all reachable hosts\n", argv[0]);
	  printf("\t%s -l\t\t\t\tList all Interfaces\n", argv[0]);
  }


  return(lRetVal);
}








/*
 *
 *
 */

void listInterfaces()
{
  pcap_if_t *lAllDevs = NULL;
  pcap_if_t *lDevice = NULL;
  char lTemp[PCAP_ERRBUF_SIZE];
  int lCounter = 0;

  /*
   * Open device list.
   *
   */

  if (pcap_findalldevs_ex(PCAP_SRC_IF_STRING, NULL, &lAllDevs, lTemp) == -1)
  {
    printf("Error in pcap_findalldevs_ex() : %s\n", lTemp);
    return;
  }



  /*
   * Print device name + description.
   *
   */

  for(lDevice = lAllDevs, lCounter = 0; lDevice; lDevice = lDevice->next, lCounter++)
  {
    printf("\n%d. %s\n    ", lCounter, lDevice->name);

    if (lDevice->name)
      printf("%d : (%s)\n", lCounter, lDevice->description);
    else
      printf(" (%d : No description available)\n", lCounter);
  }
}
