/*
 * Tool name   : ARPScanner
 * Version     : 0.1
 * Description : A tool to scan an IP range based on ARP requests/replies.
 * Author      : Ruben Unteregger
 * Webpage     : http://www.megapanzer.com
 * OS          : Tested on Microsoft Windows XP
 * Todo        : -
 *
 * Changes     : 
 * 
 *
 * License     :
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */



#define ETHERTYPE_ARP 0x0806

#define MAX_BUF_SIZE 1024
#define snprintf _snprintf

#define ETH_ALEN 6
#define IP_ALEN 4

#define ARP_REQUEST 1
#define ARP_REPLY 2




/*
 * Our data types
 *
 */

typedef struct ethern_hdr
{
  unsigned char ether_dhost[6];  // dest Ethernet address
  unsigned char ether_shost[6];  // source Ethernet address
  unsigned short ether_type;     // protocol (16-bit)
} ETHDR, *PETHDR;

typedef struct arphdr   
{   
  unsigned short  htype;   // format of hardware address 
  unsigned short  ptype;   // format of protocol address
  unsigned char   hlen;   // length of hardware address
  unsigned char   plen;    // length of protocol address 
  unsigned short  opcode;    // ARP/RARP operation
  unsigned char   sha[6];  // sender hardware address (MAC)
  unsigned char   spa[4];     // sender protocol address (IP)
  unsigned char   tha[6];  // target hardware address (MAC)
  unsigned char   tpa[4];      // target protocol address (IP)
} ARPHDR, *PARPHDR; 

typedef struct SCANPARAMS
{
  int IFCNum;
  unsigned char StartIP[4];
  unsigned char StopIP[4];
  unsigned char LocalIP[4];
  unsigned char LocalMAC[16];
} SCANPARAMS, *PSCANPARAMS;

typedef struct pARPPacket
{
  int IFCNum;
  int lReqType;
  unsigned char Eth_SrcMAC[16];
  unsigned char Eth_DstMAC[16];

  unsigned char ARP_LocalMAC[16];
  unsigned char ARP_LocalIP[4];
  unsigned char ARP_GWMac[16];
  unsigned char ARP_Dst_MAC[16];
  unsigned char ARP_DstIP[4];
} ARPPacket, *PARPPacket;

/*
 * Function forward declarations.
 *
 */

void listInterfaces();
DWORD WINAPI ScanNetwork(PSCANPARAMS lpParam);
DWORD WINAPI CaptureTargetAnswers(LPVOID lpParam);
void TargetMACSniffer_handler(u_char *param, const struct pcap_pkthdr *header, const u_char *pkt_data);
void MAC2String(unsigned char pMAC[6], char *pOutput, int pOutputLen);
int sendARPPacket(PARPPacket pARPPacket);
