/*
 * Tool name   : ARPScanner
 * Version     : 0.1
 * Description : A tool to scan an IP range based on ARP requests/replies.
 * Author      : Ruben Unteregger
 * Webpage     : http://www.megapanzer.com
 * OS          : Tested on Microsoft Windows XP
 * Todo        : -
 *
 * Changes     : 
 * 
 *
 * License     :
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#define HAVE_REMOTE

#include <stdlib.h>
#include <stdio.h>
#include <pcap.h>
#include "ARPScanner.h"


static BOOL gErrorOccurred = FALSE;

/*
 *
 *
 */

DWORD WINAPI CaptureTargetAnswers(LPVOID lpParam)
{
  DWORD lRetVal = 0;
  pcap_if_t *lAllDevs = NULL;
  pcap_if_t *lDevice = NULL;
  char lTemp[PCAP_ERRBUF_SIZE];
  char lAdapter[MAX_BUF_SIZE + 1];
  int lCounter = 0;
  int lIFCnum = 0;
  pcap_t *lIFCHandle = NULL;
  struct bpf_program lFCode;
  unsigned int lNetMask = 0;
  PSCANPARAMS lScanParams = (PSCANPARAMS) lpParam;
 

  /*
   * Open device list.
   *
   */
 
  if (pcap_findalldevs_ex(PCAP_SRC_IF_STRING, NULL, &lAllDevs, lTemp) == -1)
  {
    printf("\nListener::Error in pcap_findalldevs_ex() : %s\n", lTemp);
    gErrorOccurred = TRUE;
    lRetVal = 1;
    goto END;
  }

 
 
  ZeroMemory(lAdapter, sizeof(lAdapter));
  lCounter = 0;
 
  for(lCounter = 0, lDevice = lAllDevs; lDevice; lDevice = lDevice->next, lCounter++)
  {
    if (lCounter == lScanParams->IFCNum)
    {
      strcpy(lAdapter, lDevice->name);
      break;
    }
  }
 
 
 
  /*
   * Open interface.
   *
   */
 
  if ((lIFCHandle = pcap_open(lAdapter, 65536, PCAP_OPENFLAG_PROMISCUOUS, 1000, NULL, lTemp)) == NULL)
  {
    fprintf(stderr,"\nListener::Unable to open the adapter.\n");
    lRetVal = 5;
    gErrorOccurred = TRUE;
	goto END;
  }
 
 
 
 
 
  /* 
   * Compiling + setting the filter
   *
   */
 
  if (lDevice->addresses != NULL)
    lNetMask = ((struct sockaddr_in *)(lDevice->addresses->netmask))->sin_addr.S_un.S_addr;
  else
    lNetMask = 0xffffff; 
 
 
  ZeroMemory(&lFCode, sizeof(lFCode));
  if (pcap_compile(lIFCHandle, &lFCode, "arp", 1, lNetMask) < 0)
  {
    printf("\nListener::Unable to compile the packet filter.\n");
    lRetVal = 3;
    gErrorOccurred = TRUE;
	goto END;
  }
 
 
  if (pcap_setfilter(lIFCHandle, &lFCode) < 0)
  {
    printf("\nListener::Error setting the filter.\n");
    lRetVal = 4;
    gErrorOccurred = TRUE;
	goto END;
  }
 
  // We dont need this list anymore.
  pcap_freealldevs(lAllDevs);
 
  printf("Scanner started. Waiting for replies ...\n\n\n");
  // Start intercepting data packets.
  pcap_loop(lIFCHandle, 0, TargetMACSniffer_handler, NULL);
 
 
 
END:
 
  /*
   * Release all allocated resources.
   *
   */
 
  if (lAllDevs)
    pcap_freealldevs(lAllDevs);
 

  return(lRetVal);
}


 
/* 
 * Callback function invoked by libpcap for every incoming packet 
 *
 */
 
void TargetMACSniffer_handler(u_char *param, const struct pcap_pkthdr *header, const u_char *pkt_data)
{
  PETHDR lEHdr = (PETHDR) pkt_data;
  PARPHDR pARPData = NULL;
  char lDMAC[64];
  char lDIP[64];
  char lSMAC[64];
  char lSIP[64];

  if (htons(lEHdr->ether_type) == 0x0806 )
  {
	pARPData = (PARPHDR) (pkt_data + 14);

    if (ntohs(pARPData->opcode) == ARP_REPLY)
	{
      ZeroMemory(lDMAC, sizeof(lDMAC));
      ZeroMemory(lSMAC, sizeof(lSMAC));
      ZeroMemory(lDIP, sizeof(lDIP));
      ZeroMemory(lSIP, sizeof(lSIP)); 

      MAC2String(pARPData->sha, lSMAC, sizeof(lSMAC) - 1);
      MAC2String(pARPData->tha, lDMAC, sizeof(lDMAC) - 1);

      snprintf(lDIP, sizeof(lDIP) - 1, "%d.%d.%d.%d", pARPData->tpa[0], 
      pARPData->tpa[1], pARPData->tpa[2], pARPData->tpa[3]);

      snprintf(lSIP, sizeof(lSIP) - 1, "%d.%d.%d.%d", pARPData->spa[0], 
	  pARPData->spa[1], pARPData->spa[2], pARPData->spa[3]); 


      printf("UP\t%s / %s \n", lSMAC, lSIP);
	}
  }
}




/*
 *
 *
 */

DWORD WINAPI ScanNetwork(PSCANPARAMS lpParam) 
{
  DWORD lRetVal = 0;
  ARPPacket lARPPacket;
  PSCANPARAMS lScanParams = (PSCANPARAMS) lpParam;
  unsigned long lStartIP = 0;
  unsigned long lStopIP = 0;
  unsigned long lDstIP = 0;
  HANDLE lThreadHandle = INVALID_HANDLE_VALUE;
  DWORD lThreadId = 0;


  // Initialisation
  ZeroMemory(&lARPPacket, sizeof(lARPPacket));
  memcpy(&lStartIP, lScanParams->StartIP, 4);
  memcpy(&lStopIP, lScanParams->StopIP, 4);

  lStartIP = htonl(lStartIP);
  lStopIP = htonl(lStopIP);


  /*
   * Create and send ARP packet
   *
   */

  if (lStartIP <= lStopIP)
  {
    /*
	 * Start listener thread.
	 *
	 */

    if ((lThreadHandle = CreateThread(NULL, 0, CaptureTargetAnswers, lScanParams, 0, &lThreadId)) != NULL)
	{
      Sleep(100);

      for (; lStartIP <= lStopIP && gErrorOccurred == FALSE; lStartIP++)
      {
        lDstIP = htonl(lStartIP);
  
        lARPPacket.IFCNum = lScanParams->IFCNum;
        lARPPacket.lReqType = ARP_REQUEST;
        // Set MAC values
        memcpy(lARPPacket.Eth_SrcMAC, lScanParams->LocalMAC, 16);
        lARPPacket.Eth_DstMAC[0] = 255;
        lARPPacket.Eth_DstMAC[1] = 255;
        lARPPacket.Eth_DstMAC[2] = 255;
        lARPPacket.Eth_DstMAC[3] = 255;
        lARPPacket.Eth_DstMAC[4] = 255;
        lARPPacket.Eth_DstMAC[5] = 255;
        // Set ARP request values
        memcpy(lARPPacket.ARP_LocalMAC, lScanParams->LocalMAC, 16);
        memcpy(lARPPacket.ARP_LocalIP, lScanParams->LocalIP, 4);
        memcpy(&lARPPacket.ARP_DstIP[0], &lDstIP, 4);
        // Send packet
        sendARPPacket(&lARPPacket);

        Sleep(10);
      }

      printf("\n... Finishing scan ...\n");
      Sleep(2000);
      CloseHandle(lThreadHandle);
	}
  } else {
    printf("\nSender:: Something is wrong with the start and/or stop IP!\n\n");
  }
  
  return(lRetVal);
}



/*
 *
 *
 */
 
void MAC2String(unsigned char pMAC[6], char *pOutput, int pOutputLen)
{
  if (pOutput && pOutputLen > 0)
  {
    snprintf(pOutput, pOutputLen, "%02X:%02X:%02X:%02X:%02X:%02X", pMAC[0], 
             pMAC[1], pMAC[2], pMAC[3], pMAC[4], pMAC[5]);
  }
}




/*
 *
 *
 */

int sendARPPacket(PARPPacket pARPPacket)
{
  int lRetVal = 0;
  unsigned char lARPPacket[sizeof(ETHDR) + sizeof(ARPHDR)];
  pcap_if_t *lAllDevs = NULL;
  pcap_if_t *lDevice = NULL;
  pcap_t *lIFCHandle = NULL;
  char lTemp[PCAP_ERRBUF_SIZE];
  char lAdapter[MAX_BUF_SIZE + 1];
  int lCounter = 0;
  PETHDR lEHdr = (PETHDR) lARPPacket;
  PARPHDR lARPHdr = (PARPHDR) (lARPPacket + 14);
	  

  /*
   * Open device list.
   *
   */

  if (pcap_findalldevs_ex(PCAP_SRC_IF_STRING, NULL, &lAllDevs, lTemp) == -1)
  {
    printf("\nSender::Error in pcap_findalldevs_ex() : %s\n", lTemp);
    lRetVal = 2;
    gErrorOccurred = TRUE;
    goto END;
  }

  ZeroMemory(lAdapter, sizeof(lAdapter));
  lCounter = 0;

  for(lCounter = 0, lDevice = lAllDevs; lDevice; lDevice = lDevice->next, lCounter++)
  {
    if (lCounter == pARPPacket->IFCNum)
    {
      strcpy(lAdapter, lDevice->name);
      break;
    }
  }



  /*
   * Open interface.
   *
   */

  if ((lIFCHandle = pcap_open(lAdapter, 65536, PCAP_OPENFLAG_PROMISCUOUS, 1000, NULL, lTemp)) == NULL)
  {
    fprintf(stderr, "\nSender::Unable to open the adapter \"%s\".\n", lAdapter);
    lRetVal = 3;
    gErrorOccurred = TRUE;
	goto END;
  }


  ZeroMemory(lARPPacket, sizeof(lARPPacket));

  /*
   * Layer 1/2 (Physical)
   *
   */

  memcpy(lEHdr->ether_shost, pARPPacket->Eth_SrcMAC, 6);
  memcpy(lEHdr->ether_dhost, pARPPacket->Eth_DstMAC, 6);
  lEHdr->ether_type = htons(ETHERTYPE_ARP);


  /*
   * Layer 2
   *
   */

  lARPHdr->htype = htons(0x0001); // Ethernet
  lARPHdr->ptype = htons(0x0800); // IP
  lARPHdr->hlen = 0x0006;
  lARPHdr->plen = 0x0004;
  lARPHdr->opcode = htons(pARPPacket->lReqType);

  memcpy(lARPHdr->tpa, pARPPacket->ARP_DstIP, 4);
  memcpy(lARPHdr->tha, pARPPacket->ARP_Dst_MAC, 6);

  memcpy(lARPHdr->spa, pARPPacket->ARP_LocalIP, 4);
  memcpy(lARPHdr->sha, pARPPacket->ARP_LocalMAC, 6);

  /* Send down the packet */
  if (pcap_sendpacket(lIFCHandle, lARPPacket, sizeof(ETHDR) + sizeof(ARPHDR)) != 0)
  {
    fprintf(stderr, "\nSender::Error sending the packet: %s\n", pcap_geterr(lIFCHandle));
    lRetVal = 4;
    gErrorOccurred = TRUE;
    goto END;
  }

END:
  if (lIFCHandle)
    pcap_close(lIFCHandle);

  return(lRetVal);
}
